package pwc.taxtech.atms.controller;

import java.util.List;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.ResponseBody;
import org.springframework.web.bind.annotation.RestController;

import io.swagger.annotations.ApiOperation;
import pwc.taxtech.atms.dto.MenuDisplayDto;
import pwc.taxtech.atms.dto.MenuDto;
import pwc.taxtech.atms.service.MenuService;

/** @see PwC.Tax.Tech.Atms.WebApi\Controllers\MenuController.cs */
@RestController
@RequestMapping("/api/v1/menu")
public class MenuController {

    private static final Logger logger = LoggerFactory.getLogger(MenuController.class);

    @Autowired
    private MenuService menuService;

    @ApiOperation(value = "获取用于显示的菜单列表", notes = "请提供条件参数serviceId")
    @RequestMapping(value = "/display", method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody List<MenuDisplayDto> getMenusForDisplay(@RequestParam("serviceId") Integer serviceId) {
        logger.info("come into menu display");
        return menuService.getMenusForDisplay(serviceId);
    }

    @ApiOperation(value = "获取菜单列表", notes = "请提供条件参数serviceId")
    @RequestMapping(method = RequestMethod.GET, produces = MediaType.APPLICATION_JSON_UTF8_VALUE)
    public @ResponseBody List<MenuDto> getMenus(@RequestParam("serviceId") Integer serviceId) {
        logger.info("come into menu");
        return menuService.getMenus(serviceId);
    }
}
