

package pwc.taxtech.atms.service.impl;

import java.io.InputStream;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import org.nutz.lang.Files;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import io.jsonwebtoken.lang.Collections;
import pwc.taxtech.atms.common.CommonConstants;
import pwc.taxtech.atms.common.CommonUtils;
import pwc.taxtech.atms.common.OperateLogType;
import pwc.taxtech.atms.common.OperationAction;
import pwc.taxtech.atms.common.OperationModule;
import pwc.taxtech.atms.common.message.EnterpriseAccountMessage;
import pwc.taxtech.atms.common.message.LogMessage;
import pwc.taxtech.atms.constant.EnterpriseAccountConstant;
import pwc.taxtech.atms.constant.IndustryConstant;
import pwc.taxtech.atms.dto.OperationResultDto;
import pwc.taxtech.atms.dto.UpdateLogParams;
import pwc.taxtech.atms.dto.ValidateInfoDto;
import pwc.taxtech.atms.dto.epaccount.AccountMappingDto;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountAndValidateInfo;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountDto;
import pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetDto;
import pwc.taxtech.atms.entitiy.AccountMapping;
import pwc.taxtech.atms.entitiy.AccountMappingExample;
import pwc.taxtech.atms.entitiy.EnterpriseAccount;
import pwc.taxtech.atms.entitiy.EnterpriseAccountExample;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSet;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrg;
import pwc.taxtech.atms.entitiy.EnterpriseAccountSetOrgExample;
import pwc.taxtech.atms.entitiy.StandardAccount;
import pwc.taxtech.atms.entitiy.StandardAccountExample;
import pwc.taxtech.atms.service.AccountService;
import pwc.taxtech.atms.service.EnterpriseAccountService;
import pwc.taxtech.atms.service.EnterpriseAccountSetService;

@Service
public class EnterpriseAccountServiceImpl extends AbstractService implements EnterpriseAccountService {

    @Autowired private AccountService accountService;
    @Autowired private EnterpriseAccountSetService enterpriseAccountSetService;
    @Autowired private FileService fileService;
    
    private Map<String, List<String>> mapParentAccountResult;
    
    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#getEnterpriseAccount(java.lang.String, java.lang.String)
     */
    @Override
    public EnterpriseAccount getEnterpriseAccount(String enterpriseAccountCode, String enterpriseAccountSetID) {
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andCodeEqualTo(enterpriseAccountCode).andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
        
        List<EnterpriseAccount> epAccounts = enterpriseAccountMapper.selectByExample(example);
        return Collections.isEmpty(epAccounts)?null:epAccounts.get(0);
    }

    /*
     * (non-Javadoc)
     * 
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#
     * getListByEnterpriseAccountSetID(java.lang.String)
     */
    @Override
    public EnterpriseAccountAndValidateInfo getListByEnterpriseAccountSetID(String enterpriseAccountSetID) {

        EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
        enterpriseAccountExample.createCriteria()
            .andRuleTypeEqualTo(2)
            .andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
        enterpriseAccountExample.setOrderByClause("ParentCode ASC, IsActive DESC, Code ASC");
        List<EnterpriseAccount> epAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
        
        //no enterprise account
        if(epAccounts==null || epAccounts.isEmpty()) {
            return new EnterpriseAccountAndValidateInfo();
        }
        
        //get all standard accounts
        StandardAccountExample standardAccountExample = new StandardAccountExample();
        List<StandardAccount> stdAccounts = standardAccountMapper.selectByExample(standardAccountExample);
        
        //set stdName
        List<EnterpriseAccountDto> epAccountDtoList = new ArrayList<>();
        for(EnterpriseAccount epAccount: epAccounts) {
            EnterpriseAccountDto epAccountDto = new EnterpriseAccountDto();
            
            CommonUtils.copyProperties(epAccount, epAccountDto);
            epAccountDto.setParentName("");
            epAccountDto.setParentFullName("");
            epAccountDto.setStdName("");
            stdAccounts.stream().filter(stdAccount -> stdAccount.getCode().equals(epAccount.getStdCode())).findFirst()
                .ifPresent(stdAccount -> {epAccountDto.setStdName(stdAccount.getName());});
            epAccountDtoList.add(epAccountDto);
        }
        
        //wrap epAccounts and generate retEpAccountDtoList
        List<EnterpriseAccountDto> retEpAccountDtoList = new ArrayList<>();
        List<EnterpriseAccountDto> topEpAccountDtoList 
            = epAccountDtoList.stream().filter(epAccountDto -> epAccountDto.getParentCode() == null)
                .collect(Collectors.toList());
        for(EnterpriseAccountDto epAccountDto:  topEpAccountDtoList) {
            epAccountDto.setTreeLevel(0);
            retEpAccountDtoList.add(epAccountDto);
            retEpAccountDtoList.addAll(getWrapList(epAccountDto, epAccountDtoList));
        }
        
        //lost epAccounts list
        for(EnterpriseAccountDto epAccountDto:  epAccountDtoList) {
            if( !retEpAccountDtoList.contains(epAccountDto)) {
                retEpAccountDtoList.add(epAccountDto);
            }
        }
        
        //validate enterprise account list
        List<ValidateInfoDto> validateInfoList = validateEnterpriseAccountList(retEpAccountDtoList);
        
        EnterpriseAccountAndValidateInfo enterpriseAccountAndValidateInfo = new EnterpriseAccountAndValidateInfo();
        enterpriseAccountAndValidateInfo.setEnterpriseAccountList(retEpAccountDtoList);
        enterpriseAccountAndValidateInfo.setValidateInfoList(validateInfoList);
        
        return enterpriseAccountAndValidateInfo;
    }
    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#getEnterpriseAccount(java.lang.String)
     */
    @Override
    public EnterpriseAccountDto getEnterpriseAccount(String enterpriseAccountId) {
        
        EnterpriseAccount enterpriseAccount = enterpriseAccountMapper.selectByPrimaryKey(enterpriseAccountId);
        
        //get all enterprise accounts
        EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
        enterpriseAccountExample.createCriteria()
            .andIsActiveEqualTo(CommonConstants.ACTIVE_STATUS)
            .andRuleTypeEqualTo(2)
            .andEnterpriseAccountSetIDEqualTo(enterpriseAccount.getEnterpriseAccountSetID());
        List<EnterpriseAccount> epAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
        
        //get all standard accounts
        StandardAccountExample standardAccountExample = new StandardAccountExample();
        List<StandardAccount> stdAccounts = standardAccountMapper.selectByExample(standardAccountExample);
        
        EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
        CommonUtils.copyProperties(enterpriseAccount, enterpriseAccountDto);
        epAccounts.stream().filter(epAccount -> enterpriseAccount.getParentCode()!=null && enterpriseAccount.getParentCode().equals(epAccount.getCode())).findFirst()
            .ifPresent(epAccount -> {
                enterpriseAccountDto.setParentName(epAccount.getName());
                enterpriseAccountDto.setParentFullName(epAccount.getFullName());
                });
        stdAccounts.stream().filter(stdAccount -> stdAccount.getCode().equals(enterpriseAccount.getStdCode())).findFirst()
            .ifPresent(stdAccount -> {
                enterpriseAccountDto.setStdName(stdAccount.getName());
                });
        enterpriseAccountDto.setSubStdAccounts(new ArrayList<>());
        enterpriseAccountDto.setIndustryID("");
        
        return enterpriseAccountDto;
    }
    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#addEnterpriseAccount(pwc.taxtech.atms.service.dto.EnterpriseAccountDto)
     */
    @Override
    @Transactional
    public OperationResultDto<List<EnterpriseAccountDto>> addEnterpriseAccount(EnterpriseAccountDto enterpriseAccountDto) {

        if(enterpriseAccountDto == null) {
            return new OperationResultDto<>(false);
        }
        EnterpriseAccount entityToAdd = new EnterpriseAccount();
        CommonUtils.copyProperties(enterpriseAccountDto, entityToAdd);
        
        //check duplication ep account
        List<EnterpriseAccountDto> sameEpAccounts = getSameEnterpriseAccountList(enterpriseAccountDto);
        if(sameEpAccounts!=null && !sameEpAccounts.isEmpty()) {
            return new OperationResultDto<>(false, EnterpriseAccountMessage.EnterpriceAccountRepeat, sameEpAccounts);
        }
        //set parent account no leaf
        setParentIsNotLeaf(enterpriseAccountDto.getParentCode(), enterpriseAccountDto.getEnterpriseAccountSetID());
        //add enterprise accoun
        entityToAdd.setID(CommonUtils.getUUID());
        entityToAdd.setIsLeaf(true);
        entityToAdd.setCreateTime(new Date());
        entityToAdd.setUpdateTime(new Date());
        entityToAdd.setCreatorID(authUserHelper.getCurrentUserID());
        entityToAdd.setUpdatorID(authUserHelper.getCurrentUserID());
        enterpriseAccountMapper.insert(entityToAdd);
        
        //获取与之关联的机构
        EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountDto.getEnterpriseAccountSetID());
        List<EnterpriseAccountSetOrg> accountSetOrgList = enterpriseAccountSetOrgMapper.selectByExampleWithAssociation(example);
        
        //构造数据结构，调用更新父级科目对应方法
        List<String> epAccountCodeList = new ArrayList<>();
        epAccountCodeList.add(entityToAdd.getCode());
        
        this.mapParentAccountResult = new HashMap<>();
        for(EnterpriseAccountSetOrg accountSetOrg : accountSetOrgList) {
            mapAccountUpdateParent(epAccountCodeList, accountSetOrg.getEnterpriseAccountSetID(), getConvertedIndustryID(accountSetOrg.getOrganization().getIndustryID()), accountSetOrg.getOrganizationID(), true);
        }
        
        //get enterprise set name
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey(enterpriseAccountDto.getEnterpriseAccountSetID());
        //add operational log
        UpdateLogParams updateLogParams = new UpdateLogParams();
        updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor());
        updateLogParams.setUpdateState("");
        updateLogParams.setOriginalState("");
        updateLogParams.setOperationContent(enterpriseAccountDto.getCode() + " " + enterpriseAccountDto.getName());
        updateLogParams.setOperationObject(enterpriseAccountSet.getName());
        updateLogParams.setOperationModule(OperationModule.BasicDataEnterpriceAccount.value());
        updateLogParams.setOperateLogType(OperateLogType.OperationLogEnterPrise.value());
        updateLogParams.setComment("");
        updateLogParams.setOperationAction(OperationAction.New.value());
        operationLogService.addOrDeleteDataAddLog(updateLogParams);
        
        return new OperationResultDto<>(true);
    }

    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#updateEnterpriseAccount(pwc.taxtech.atms.service.dto.EnterpriseAccountDto)
     */
    @Override
    @Transactional
    public OperationResultDto<List<EnterpriseAccountDto>> updateEnterpriseAccount(EnterpriseAccountDto enterpriseAccountDto) {

        if(enterpriseAccountDto == null) {
            return new OperationResultDto<>(false);
        }
        
        EnterpriseAccount entityToUpdate = enterpriseAccountMapper.selectByPrimaryKey(enterpriseAccountDto.getID());
        if(entityToUpdate == null) {
            return new OperationResultDto<>(false);
        }
        
        //check duplication ep account
        List<EnterpriseAccountDto> sameEpAccounts = getSameEnterpriseAccountList(enterpriseAccountDto);
        if(sameEpAccounts!=null && !sameEpAccounts.isEmpty()) {
            return new OperationResultDto<>(false, EnterpriseAccountMessage.EnterpriceAccountRepeat, sameEpAccounts);
        }
        
        //copy entityToUpdate
        EnterpriseAccount entityCopy = new EnterpriseAccount();
        CommonUtils.copyProperties(entityToUpdate, entityCopy);
        
        if(entityCopy.getParentCode()!=null && !entityCopy.getParentCode().equals(enterpriseAccountDto.getParentCode())) {
            // reset the isleaf
            setParentIsNotLeaf(enterpriseAccountDto.getParentCode(), enterpriseAccountDto.getEnterpriseAccountSetID());
            setParentLeaf(entityCopy.getParentCode(), enterpriseAccountDto.getEnterpriseAccountSetID(), enterpriseAccountDto.getID());
        }
        
        
        //查询所有原本对应子科目
        //如修改的为父科目, 需要级联修改其所有子科目的父科目代码以及名字
        EnterpriseAccountExample enterpriseAccountexample = new EnterpriseAccountExample();
        enterpriseAccountexample.createCriteria()
            .andParentCodeEqualTo(entityCopy.getCode())
            .andEnterpriseAccountSetIDEqualTo(enterpriseAccountDto.getEnterpriseAccountSetID());
        List<EnterpriseAccount> childrenAccountList = enterpriseAccountMapper.selectByExample(enterpriseAccountexample);
        if(childrenAccountList!=null && !childrenAccountList.isEmpty()) {
            if(!entityCopy.getCode().equals(enterpriseAccountDto.getCode()) || !entityCopy.getName().equals(enterpriseAccountDto.getName())) {
                for(EnterpriseAccount childrenAccount: childrenAccountList) {
                    childrenAccount.setParentCode(enterpriseAccountDto.getCode());
                    childrenAccount.setFullName(enterpriseAccountDto.getFullName() + EnterpriseAccountConstant.FullNameSeparator + childrenAccount.getName());
                    enterpriseAccountMapper.updateByPrimaryKey(childrenAccount);
                }
            }
        }
        
        entityToUpdate.setCode(enterpriseAccountDto.getCode());
        entityToUpdate.setName(enterpriseAccountDto.getName());
        entityToUpdate.setFullName(enterpriseAccountDto.getFullName());
        entityToUpdate.setParentCode(enterpriseAccountDto.getParentCode());
        entityToUpdate.setDirection(enterpriseAccountDto.getDirection());
        entityToUpdate.setAcctProp(enterpriseAccountDto.getAcctProp());
        entityToUpdate.setStdCode(enterpriseAccountDto.getStdCode());
        enterpriseAccountMapper.updateByPrimaryKey(entityToUpdate);
        
        //获取与之关联的机构
        EnterpriseAccountSetOrgExample example = new EnterpriseAccountSetOrgExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountDto.getEnterpriseAccountSetID());
        List<EnterpriseAccountSetOrg> accountSetOrgList = enterpriseAccountSetOrgMapper.selectByExampleWithAssociation(example);
        
        //构造数据结构，调用更新父级科目对应方法
        List<String> epAccountCodeList = new ArrayList<>();
        epAccountCodeList.add(entityToUpdate.getCode());
        
        this.mapParentAccountResult = new HashMap<>();
        for(EnterpriseAccountSetOrg accountSetOrg : accountSetOrgList) {
            if(entityToUpdate.getParentCode()!=null && entityToUpdate.getParentCode().equals(entityCopy.getParentCode())) {
                
                /* @See below C#代码 EnterpriseAccountService.cs Line 238-240
                 * string updateSql = "UPDATE AccountMapping SET StandardAccountCode='00' WHERE EnterpriseAccountCode={0} AND EnterpriseAccountSetID={1} AND OrganizationID={2} AND IndustryID={3}";
                   _dbContext.Database.ExecuteSqlCommand(updateSql, entity.ID, item.EnterpriseAccountSetID, item.OrganizationID, GetConvertedIndustryID(item.IndustryID));
                 * EnterpriseAccountCode 参数应该设错了
                 */
                
                //需要清除旧的对应关系
                AccountMapping accountMapping = new AccountMapping();
                accountMapping.setStandardAccountCode("00");
                AccountMappingExample accountMappingExample = new AccountMappingExample();
                accountMappingExample.createCriteria()
                    .andEnterpriseAccountCodeEqualTo(entityCopy.getParentCode())
                    .andEnterpriseAccountSetIDEqualTo(accountSetOrg.getEnterpriseAccountSetID())
                    .andOrganizationIDEqualTo(accountSetOrg.getOrganizationID())
                    .andIndustryIDEqualTo(getConvertedIndustryID(accountSetOrg.getOrganization().getIndustryID()));
                accountMappingMapper.updateByExampleSelective(accountMapping, accountMappingExample);
            }
            mapAccountUpdateParent(epAccountCodeList, accountSetOrg.getEnterpriseAccountSetID(), 
                     getConvertedIndustryID(accountSetOrg.getOrganization().getIndustryID()), accountSetOrg.getOrganizationID(), true);
        }
        //get enterprise set name
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey(enterpriseAccountDto.getEnterpriseAccountSetID());
        //operation log
        UpdateLogParams updateLogParams = new UpdateLogParams();
        updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor());
        updateLogParams.setUpdateState(getEnterpriseAccountInfo(entityToUpdate));
        updateLogParams.setOriginalState(getEnterpriseAccountInfo(entityCopy));
        updateLogParams.setOperationContent(enterpriseAccountDto.getCode() + " " + enterpriseAccountDto.getName());
        updateLogParams.setOperationObject(enterpriseAccountSet.getName());
        updateLogParams.setOperationModule(OperationModule.BasicDataEnterpriceAccount.value());
        updateLogParams.setOperateLogType(OperateLogType.OperationLogEnterPrise.value());
        updateLogParams.setComment("");
        updateLogParams.setOperationAction(OperationAction.UpdateEnterpriseAccount.value());
        operationLogService.addOrDeleteDataAddLog(updateLogParams);
        
        enableEnterpriseAccountSet(enterpriseAccountDto.getEnterpriseAccountSetID());
        
        return new OperationResultDto<>(true);
    }
    
    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#mapAccountUpdateParent(java.util.List, java.lang.String, java.lang.String, java.lang.String, java.lang.Boolean)
     */
    @Override
    public void mapAccountUpdateParent(List<String> enterpriseAccountCodeList, String enterpriseAccountSetID,
            String industryID, String orgID, Boolean isToUpdateEvenExists) {

        for (String epAccountCode : enterpriseAccountCodeList) {
            
            /* 数据准备 start */
            String mappedStdCode = "";
            
            //第一步:通过企业科目Code获取企业科目对象
            EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
            enterpriseAccountExample.createCriteria()
                .andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID)
                .andCodeEqualTo(epAccountCode);
            List<EnterpriseAccount> epAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
            if(Collections.isEmpty(epAccounts)) {
                continue;
            }
            EnterpriseAccount epAccount = epAccounts.get(0);
            //如果当前对应的本身就是第一级，不需要再考虑
            if(epAccount.getParentCode() == null || epAccount.getParentCode().isEmpty()) {
                continue;
            }
            
            //第二步: 通过上一步获取到的企业科目对象的ParentCode获取企业企业科目对象
            enterpriseAccountExample = new EnterpriseAccountExample();
            enterpriseAccountExample.createCriteria()
                .andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID)
                .andCodeEqualTo(epAccount.getParentCode());
            List<EnterpriseAccount> parentEpAccounts = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
            if(Collections.isEmpty(parentEpAccounts)) {
                return; //C# return
            }
            EnterpriseAccount parentEpAccount = parentEpAccounts.get(0);
            
            //第三步: 通过Parent企业科目对象获取该ParentCode下面的所有子企业科目
            List<EnterpriseAccount> childrenEpAccountList = getMappingResultByParentCode(parentEpAccount.getCode(), enterpriseAccountSetID, industryID, orgID);
            
            //第四步: 判断第三步获取到企业科目中,标准科目为空的个数
            if(Collections.isEmpty(childrenEpAccountList)) {
                continue;
            }
            long emptyStdCodeCount = childrenEpAccountList.stream().filter(sa -> sa.getStdCode()==null || sa.getStdCode().isEmpty()).count();
            
            //第五部: 将第三步获取到的所有企业科目取出其中的StdCode并去重
            List<String> uniqueStdCodeList = childrenEpAccountList.stream()
                    .filter(sa -> sa.getStdCode()!=null && !sa.getStdCode().isEmpty())
                    .map(sa -> sa.getStdCode()).collect(Collectors.toList());
            /* 数据准备 end */
            
            /* 业务处理 start */
            
            //等于0说明子科目全部都有对应上
            if(emptyStdCodeCount == 0) {
                
                long countOfDistinctStdCode = uniqueStdCodeList.size();
                
                //等于1说明对应上的标准科目都是同一个
                if(countOfDistinctStdCode == 1) {
                    //2. 手动对应，子科目完全对应的时候并且都对应到同一个标准科目，那么父级也要对应到该标准科目
                    mappedStdCode = uniqueStdCodeList.get(0);
                }
                //说明虽然完全对应上了，但是对应上的标准科目代码不唯一
                else {
                    
                    //3. 手动对应，子科目完全对应的时候但是没有对应到同一个标准科目
                    StandardAccountExample standardAccountExample = new StandardAccountExample();
                    standardAccountExample.createCriteria()
                        .andCodeIn(uniqueStdCodeList).andIndustryIDEqualTo(industryID);
                    List<StandardAccount> standardAccountList = standardAccountMapper.selectByExample(standardAccountExample);
                    if(Collections.isEmpty(standardAccountList)) {
                        continue;
                    }
                    List<String> distinctStdParentCodeList = standardAccountList.stream()
                            .map(sa -> sa.getParentCode()).distinct().collect(Collectors.toList());
                    long uniqueStdParentCodeCount = distinctStdParentCodeList.size();
                    
                    //等于1说明对应上的都属于同一个父级标准科目
                    if(uniqueStdParentCodeCount == 1 
                            && distinctStdParentCodeList.get(0)!=null && !distinctStdParentCodeList.get(0).isEmpty() 
                            && !uniqueStdCodeList.contains("0000")) {
                        //子科目完全对应的时候但是没有对应到同一个标准科目但是属于同一个父科目下，那么该企业科目的父级也要对应到该标准科目的父级
                        mappedStdCode = standardAccountList.get(0).getParentCode();
                    }
                    else {
                        //子科目完全对应的时候但是没有对应到同一个标准科目又不属于同一个父科目下，那么该企业科目的父级显示为横线 -
                        mappedStdCode = CommonConstants.EmptyStdCode;
                    }
                }
            }
            else {
                //4.  手动对应，子科目没有全对应的时候，那么父级科目显示为未对应
                mappedStdCode = null;
            }
            
            /* 更新UI准备  start */
            StandardAccount stdAccount = new StandardAccount();
            if(mappedStdCode == CommonConstants.EmptyStdCode) {
                stdAccount.setCode(CommonConstants.EmptyStdCode);
                stdAccount.setName("");
            }
            else if(mappedStdCode == null) {
                stdAccount.setCode(null);
                stdAccount.setName("");
            }
            else {
                StandardAccountExample standardAccountExample = new StandardAccountExample();
                standardAccountExample.createCriteria()
                    .andCodeEqualTo(mappedStdCode)
                    .andIndustryIDEqualTo(industryID);
                List<StandardAccount> stdAccountList = standardAccountMapper.selectByExample(standardAccountExample);
                if (Collections.isEmpty(stdAccountList)) {
                    continue;
                }
                stdAccount = stdAccountList.get(0);
            }
            addMapParentAccountResult(parentEpAccount.getCode(), Arrays.asList(stdAccount.getCode(), stdAccount.getName()));
            
            //更新该企业科目的标准科目为mappedStdCode
            accountService.mapStdAccountByCode(enterpriseAccountSetID, stdAccount.getCode(), epAccountCode, industryID, orgID, isToUpdateEvenExists, 0);
            
            /* 更新UI准备  end */
            /* 业务处理 end */
            
            /* 如果还存在上一级的情况，所以递归处理 start */
            mapAccountUpdateParent(Arrays.asList(parentEpAccount.getCode()), enterpriseAccountSetID, industryID, orgID, isToUpdateEvenExists);
            /* 如果还存在上一级的情况，所以递归处理 end */
        }
    }

    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#createEnterpriseAccountSetAndimportEnterpriseAccount(java.lang.String, java.lang.String, java.util.List)
     */
    @SuppressWarnings("rawtypes")
    @Override
    @Transactional
    public OperationResultDto addEnterpriseAccountSetAndImportData(String enterpriseAccountSetName,
            String enterpriseAccountSetCode, List<EnterpriseAccountDto> importData) {
        
        EnterpriseAccountSetDto enterpriseAccountSetDto = new EnterpriseAccountSetDto();
        enterpriseAccountSetDto.setName(enterpriseAccountSetName);
        enterpriseAccountSetDto.setCode(enterpriseAccountSetCode);
        //validate name is duplicated
        OperationResultDto<?> nameValidate = enterpriseAccountSetService.enterpriseAccountSetNameValidate(enterpriseAccountSetDto);
        if(!nameValidate.getResult()) {
            return nameValidate;
        }
        //validate code is duplicated
        OperationResultDto<?> codeValidate = enterpriseAccountSetService.enterpriseAccountSetCodeValidate(enterpriseAccountSetDto);
        if(!codeValidate.getResult()) {
            return codeValidate;
        }
        
        //save enterprise account set
        EnterpriseAccountSet enterpriseAccountSet = new EnterpriseAccountSet();
        CommonUtils.copyProperties(enterpriseAccountSetDto, enterpriseAccountSet);
        enterpriseAccountSet.setID(CommonUtils.getUUID());
        enterpriseAccountSet.setIsActive(CommonConstants.ACTIVE_STATUS);
        enterpriseAccountSet.setCreateTime(new Date());
        enterpriseAccountSet.setUpdateTime(new Date());
        enterpriseAccountSet.setCreatorID(authUserHelper.getCurrentUserID());
        enterpriseAccountSetMapper.insert(enterpriseAccountSet);
        
        //create operationlog for creating enterprise account set
        UpdateLogParams updateLogParams = new UpdateLogParams();
        updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor());
        updateLogParams.setUpdateState("");
        updateLogParams.setOriginalState("");
        updateLogParams.setOperationContent("");
        updateLogParams.setOperationModule(OperationModule.BasicDataEnterpriceAccount.value());
        updateLogParams.setComment("");
        updateLogParams.setOperateLogType(OperateLogType.OperationLogEnterPrise.value());
        updateLogParams.setOperationAction(OperationAction.AddEnterpriseAccountSet.value());
        updateLogParams.setOperationObject(enterpriseAccountSet.getName());
        operationLogService.addOrDeleteDataAddLog(updateLogParams);
        
        //nothing to import
        if(importData == null && Collections.isEmpty(importData)) {
            return OperationResultDto.success();
        }
        
        importDataProcess(importData, enterpriseAccountSet.getID(), true);
        
        return OperationResultDto.success();
    }
    

    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#repeatImportEnterpriseAccountSet(pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetDto, java.util.List)
     */
    @SuppressWarnings("rawtypes")
    @Override
    @Transactional
    public OperationResultDto repeatImportEnterpriseAccountSet(EnterpriseAccountSetDto enterpriseAccountSetDto,
            List<EnterpriseAccountDto> importData) {

        return importDataProcess(importData, enterpriseAccountSetDto.getID(), enterpriseAccountSetDto.getIsImportAppend());
    }

    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#clearRepeatEnterpriseAccountList(pwc.taxtech.atms.dto.epaccount.EnterpriseAccountSetDto)
     */
    @SuppressWarnings("rawtypes")
    @Override
    public OperationResultDto clearRepeatEnterpriseAccountList(EnterpriseAccountSetDto enterpriseAccountSetDto) {
        
        EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
        enterpriseAccountExample.createCriteria()
            .andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetDto.getID());
        List<EnterpriseAccount> epAccountList = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
        
        if(enterpriseAccountSetDto.getRepeatCodeList() == null && enterpriseAccountSetDto.getRepeatCodeList().isEmpty()) {
            return OperationResultDto.success();
        }
        
        //删除重复项 留1个
        for(String repeatCode: enterpriseAccountSetDto.getRepeatCodeList()) {
            List<EnterpriseAccount> repeatedEpAccountList = epAccountList.stream()
                    .filter(sa -> sa.getCode().equals(repeatCode)).collect(Collectors.toList());
            for(int i=0; i<repeatedEpAccountList.size()-1; i++) {
                enterpriseAccountMapper.deleteByPrimaryKey(repeatedEpAccountList.get(i).getID());
            }
        }
        
        enableEnterpriseAccountSet(enterpriseAccountSetDto.getID());
        
        return OperationResultDto.success();
    }

    /**
     * Generate by-level wrapped enterprise Account list
     * @param currentEpAccountDto
     * @param epAccountDtoList
     * @return List<EnterpriseAccountDto>
     */
    private List<EnterpriseAccountDto> getWrapList(EnterpriseAccountDto currentEpAccountDto, List<EnterpriseAccountDto> epAccountDtoList) {
        
        List<EnterpriseAccountDto> result = new ArrayList<>();
        List<EnterpriseAccountDto> subList = epAccountDtoList.stream()
                .filter(epAccountDto -> currentEpAccountDto.getCode().equals(epAccountDto.getParentCode()))
                .collect(Collectors.toList());
        
        if(subList!=null && !subList.isEmpty()) {
            for(EnterpriseAccountDto sub: subList) {
                sub.setTreeLevel(currentEpAccountDto.getTreeLevel()+1);
                sub.setParentName(currentEpAccountDto.getName());
                sub.setParentFullName(currentEpAccountDto.getFullName());
                result.add(sub);
                List<EnterpriseAccountDto> tempList = getWrapList(sub, epAccountDtoList);
                if(tempList!=null && !tempList.isEmpty()) {
                    result.addAll(tempList);
                }
            }
        }
        return result;
    }
    
    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#getAccountMappingOrg(java.lang.String)
     */
    @Override
    public List<AccountMappingDto> getAccountMappingOrg(String organizationID) {

        AccountMappingExample accountMappingExample = new AccountMappingExample();
        accountMappingExample.createCriteria()
            .andOrganizationIDEqualTo(organizationID);
        List<AccountMapping> accountMappingList = accountMappingMapper.selectByExample(accountMappingExample);
        
        List<AccountMappingDto> accountMappingDtoList = new ArrayList<>();
        accountMappingList.stream().forEach(accountMapping -> {
            AccountMappingDto accountMappingDto = new AccountMappingDto();
            accountMappingDto.setIndustryID(accountMapping.getIndustryID());
            accountMappingDto.setOrganizationID(accountMapping.getOrganizationID());
            accountMappingDtoList.add(accountMappingDto);
        });
        return accountMappingDtoList;
    }

    /**
     * Validates the enterprise account list.
     * @param epAccountList
     * @return The validate list
     */
    private List<ValidateInfoDto> validateEnterpriseAccountList(List<EnterpriseAccountDto> epAccountList) {
        List<ValidateInfoDto> validateInfoDtolist = new ArrayList<>();
        validateInfoDtolist.add(this.getRepeatValidate(epAccountList));
        validateInfoDtolist.add(this.getEmptyNameValidate(epAccountList));
        validateInfoDtolist.add(this.getDirectionValidate(epAccountList));
        validateInfoDtolist.add(this.getAcctPropNullValidate(epAccountList));
        validateInfoDtolist.add(this.getAcctPropValidate(epAccountList));
        validateInfoDtolist.add(this.getNoParentValidate(epAccountList));
        
        return validateInfoDtolist.stream()
                .filter(validateInfo -> validateInfo.getInValidateCodeList()!=null && !validateInfo.getInValidateCodeList().isEmpty())
                .collect(Collectors.toList());
        
    }
    
    /**
     * Validates if there is any duplication account code in the list
     * @param list
     * @return ValidateInfoDto
     */
    private ValidateInfoDto getRepeatValidate(List<EnterpriseAccountDto> epAccountList) {
        
        ValidateInfoDto validateInfoDto = new ValidateInfoDto();
        validateInfoDto.setType(EnterpriseAccountMessage.EnterpriceAccountRepeat);
        
        List<String> distinctCodeList = epAccountList.stream()
                .map(epAccount -> epAccount.getCode()).distinct().collect(Collectors.toList());
        
        for(String code: distinctCodeList) {
            List<EnterpriseAccountDto> accountsWithSameCode = epAccountList.stream()
                    .filter(epAccount -> code.equals(epAccount.getCode())).collect(Collectors.toList());
            if(accountsWithSameCode!=null && accountsWithSameCode.size()>1) {
                validateInfoDto.addInValidateCode(code);
            }
        }
        return validateInfoDto;
    }
    
    /**
     * Validates if there is any account with wrong direction value 
     * @param epAccountList
     * @return ValidateInfoDto
     */
    private ValidateInfoDto getDirectionValidate(List<EnterpriseAccountDto> epAccountList) {
        
        ValidateInfoDto validateInfoDto = new ValidateInfoDto();
        validateInfoDto.setType(EnterpriseAccountMessage.DirectionFormatError);
        
        List<String> inValidateCodeList = epAccountList.stream()
                .filter(epAccount -> 
                    !CommonConstants.CreditDirectionValue.equals(epAccount.getDirection()) && !CommonConstants.DebitDirectionValue.equals(epAccount.getDirection()))
                .map(epAccount -> epAccount.getCode())
                .distinct().collect(Collectors.toList());
        validateInfoDto.setInValidateCodeList(inValidateCodeList);
        
        return validateInfoDto;
    }
    
    /**
     * Validates if there is any account with wrong accprop value
     * @param epAccountList
     * @return ValidateInfoDto
     */
    private ValidateInfoDto getAcctPropValidate(List<EnterpriseAccountDto> epAccountList) {
     
        ValidateInfoDto validateInfoDto = new ValidateInfoDto();
        validateInfoDto.setType(EnterpriseAccountMessage.AcctPropFormatError);
        
        List<String> inValidateCodeList = epAccountList.stream()
                .filter(epAccount -> 
                   epAccount.getAcctProp()!=null && (epAccount.getAcctProp() > EnterpriseAccountConstant.MaxAcctPropValue || (epAccount.getAcctProp() < EnterpriseAccountConstant.MinAcctPropValue && epAccount.getAcctProp() != EnterpriseAccountConstant.SpecialAcctPropValue)))
                .map(epAccount -> epAccount.getCode())
                .distinct().collect(Collectors.toList());
        validateInfoDto.setInValidateCodeList(inValidateCodeList);
        
        return validateInfoDto;
    }
    
    /**
     * Validates if there is any account with null accprop value
     * @param epAccountList
     * @return ValidateInfoDto
     */
    private ValidateInfoDto getAcctPropNullValidate(List<EnterpriseAccountDto> epAccountList) {
        
        ValidateInfoDto validateInfoDto = new ValidateInfoDto();
        validateInfoDto.setType(EnterpriseAccountMessage.AcctPropNullError);
        
        List<String> inValidateCodeList = epAccountList.stream()
                .filter(epAccount -> 
                   epAccount.getAcctProp()==null)
                .map(epAccount -> epAccount.getCode())
                .distinct().collect(Collectors.toList());
        validateInfoDto.setInValidateCodeList(inValidateCodeList);
        
        return validateInfoDto;
    }
    
    /**
     * Validates if there is any account with empty name
     * @param epAccountList
     * @return ValidateInfoDto
     */
    private ValidateInfoDto getEmptyNameValidate(List<EnterpriseAccountDto> epAccountList) {
        
        ValidateInfoDto validateInfoDto = new ValidateInfoDto();
        validateInfoDto.setType(EnterpriseAccountMessage.EnterpriseAccountNameEmpty);
        
        List<String> inValidateCodeList = epAccountList.stream()
                .filter(epAccount -> epAccount.getName() == null || epAccount.getName().equals(""))
                .map(epAccount -> epAccount.getCode())
                .distinct().collect(Collectors.toList());
        validateInfoDto.setInValidateCodeList(inValidateCodeList);
        
        return validateInfoDto;
    }
    
    /**
     * Validates if there is any account with empty name
     * @param epAccountList
     * @return ValidateInfoDto
     */
    private ValidateInfoDto getNoParentValidate(List<EnterpriseAccountDto> epAccountList) {
        
        ValidateInfoDto validateInfoDto = new ValidateInfoDto();
        validateInfoDto.setType(EnterpriseAccountMessage.NoParentCode);
        
        List<EnterpriseAccountDto> accountsWithParent = epAccountList.stream().filter(epAccount -> epAccount.getParentCode()!=null && !epAccount.getParentCode().equals(""))
                .collect(Collectors.toList());
        
        for (EnterpriseAccountDto accountWithParent: accountsWithParent) {
            boolean parentExist = epAccountList.stream()
                .anyMatch(epAccount -> epAccount.getCode().equals(accountWithParent.getParentCode()));
            if(!parentExist) {
                validateInfoDto.addInValidateCode(accountWithParent.getCode());
            }
        }
        return validateInfoDto;
        
    }
    
    
    /**
     * Gets the same enterprise account list.
     * @param enterpriseAccountDto
     * @return
     */
    private List<EnterpriseAccountDto> getSameEnterpriseAccountList(EnterpriseAccountDto enterpriseAccountDto){
        
        if (enterpriseAccountDto == null) {
            return null;
        }
        
        EnterpriseAccountExample enterpriseAccountExample = new EnterpriseAccountExample();
        EnterpriseAccountExample.Criteria criteria = enterpriseAccountExample.createCriteria();
        if(enterpriseAccountDto.getID()!=null && !enterpriseAccountDto.getID().isEmpty()) {
            criteria.andIDNotEqualTo(enterpriseAccountDto.getID());
        }
        criteria.andEnterpriseAccountSetIDEqualTo(enterpriseAccountDto.getEnterpriseAccountSetID())
            .andCodeEqualTo(enterpriseAccountDto.getCode())
            .andIsActiveEqualTo(CommonConstants.ACTIVE_STATUS);
        List<EnterpriseAccount> epAccountsWithSameCode = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
        //没有重复项
        if(epAccountsWithSameCode == null || epAccountsWithSameCode.isEmpty()) {
            return null;
        }
        
        //有重复项
        List<EnterpriseAccountDto> sameEnterpriseAccountDtoList = new ArrayList<>();
        for(EnterpriseAccount enterpriseAccount : epAccountsWithSameCode) {
            enterpriseAccountExample = new EnterpriseAccountExample();
            criteria = enterpriseAccountExample.createCriteria();
            if(enterpriseAccount.getParentCode()!=null && !enterpriseAccount.getParentCode().isEmpty()) {
                criteria.andCodeEqualTo(enterpriseAccount.getParentCode());
            }
            criteria.andRuleTypeEqualTo(2)    
                .andIsActiveEqualTo(CommonConstants.ACTIVE_STATUS);
            List<EnterpriseAccount> epAccountsParent = enterpriseAccountMapper.selectByExample(enterpriseAccountExample);
            if(epAccountsParent!=null&&!epAccountsParent.isEmpty()) {
                epAccountsParent.stream().forEach(epAccountParent -> {
                    EnterpriseAccountDto epAccountDto = new EnterpriseAccountDto();
                    CommonUtils.copyProperties(enterpriseAccount, epAccountDto);
                    epAccountDto.setParentName(epAccountParent.getName());
                    epAccountDto.setParentFullName(epAccountParent.getFullName());
                    epAccountDto.setSubStdAccounts(new ArrayList<>());
                    sameEnterpriseAccountDtoList.add(epAccountDto);
                });
            }
            else {
                EnterpriseAccountDto epAccountDto = new EnterpriseAccountDto();
                CommonUtils.copyProperties(enterpriseAccount, epAccountDto);
                epAccountDto.setParentName("");
                epAccountDto.setParentFullName("");
                epAccountDto.setSubStdAccounts(new ArrayList<>());
                sameEnterpriseAccountDtoList.add(epAccountDto);
            }
        }
        return sameEnterpriseAccountDtoList;
    }
    
    /**
     * Sets the parent is not leaf.
     * @param parentCode - The parent code
     * @param enterpriseAccountSetID - The enterprise account set identifier
     */
    private void setParentIsNotLeaf(String parentCode, String enterpriseAccountSetID) {
        
        // no parent, do nothing
        if(parentCode == null || parentCode.isEmpty()) {
            return;
        }
        
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andCodeEqualTo(parentCode).andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
        List<EnterpriseAccount> parentNodes = enterpriseAccountMapper.selectByExample(example);
        if(parentNodes==null || parentNodes.isEmpty()) {
            return;
        }
        EnterpriseAccount parentNode = parentNodes.get(0);
        if(parentNode!=null && parentNode.getIsLeaf().equals(CommonConstants.IsLeaf)) {
            //save enterpriseAccount
            EnterpriseAccount parentNodeOriginal = new EnterpriseAccount();
            CommonUtils.copyProperties(parentNode, parentNodeOriginal);
            parentNode.setIsLeaf(CommonConstants.IsNotLeaf);
            enterpriseAccountMapper.updateByPrimaryKey(parentNode);
            //save operational log
            UpdateLogParams updateLogParams = new UpdateLogParams();
            updateLogParams.setOriginalState(parentNodeOriginal);
            updateLogParams.setUpdateState(parentNode);
            updateLogParams.setOperationModule(OperationModule.BasicDataEnterpriceAccount.value());
            updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor());
            updateLogParams.setComment(LogMessage.EnterpriceAccountImportUpdate);
            updateLogParams.setOperationObject(parentNodeOriginal.getName());
            updateLogParams.setOperationContent("operation content");
            updateLogParams.setOperateLogType(OperateLogType.OperationLogEnterPrise.value());
            updateLogParams.setOperationAction(OperationAction.Update.value());
            operationLogService.updateDataAddLog(updateLogParams);
        }
    }
    
    /**
     * Sets the parent leaf.
     * @param parentCode - The parentcode
     * @param enterpriseAccountSetID 
     * @param currentEpAccountID - Current enterprise account ID
     */
    private void setParentLeaf(String parentCode, String enterpriseAccountSetID, String currentEpAccountID) {
        // no parent, do nothing
        if(parentCode == null || parentCode.isEmpty()) {
            return;
        }
        
        EnterpriseAccount parentNodeEntity = getEnterpriseAccount(parentCode, enterpriseAccountSetID);
        if(parentNodeEntity == null) {
            return;
        }
        
        //copy parentNodeEntity
        EnterpriseAccount parentNodeOriginal = new EnterpriseAccount();
        CommonUtils.copyProperties(parentNodeEntity, parentNodeOriginal);
        
        //get other children nodes of this parent
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria()
            .andParentCodeEqualTo(parentNodeEntity.getCode())
            .andIDNotEqualTo(currentEpAccountID)
            .andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
        List<EnterpriseAccount> childrenAccountsOfparentNode = enterpriseAccountMapper.selectByExample(example);
        if(Collections.isEmpty(childrenAccountsOfparentNode)) {
            // no sub node list, set is the leaf node
            parentNodeEntity.setIsLeaf(CommonConstants.IsLeaf);
            enterpriseAccountMapper.updateByPrimaryKey(parentNodeEntity);
            
            //operation log
            UpdateLogParams updateLogParams = new UpdateLogParams();
            updateLogParams.setOriginalState(parentNodeOriginal);
            updateLogParams.setUpdateState(parentNodeEntity);
            updateLogParams.setOperationModule(OperationModule.BasicDataEnterpriceAccount.value());
            updateLogParams.setOperationUser(authUserHelper.getCurrentAuditor());
            updateLogParams.setComment(LogMessage.EnterpriceAccountImportUpdate);
            updateLogParams.setOperationObject(parentNodeOriginal.getName());
            updateLogParams.setOperationContent("operation content");
            updateLogParams.setOperateLogType(OperateLogType.OperationLogEnterPrise.value());
            updateLogParams.setOperationAction(OperationAction.Update.value());
            operationLogService.updateDataAddLog(updateLogParams);
        }
    }
    
    /**
     * 通过Parent企业科目对象获取该ParentCode下面的所有子企业科目
     * @param parentCode - Parent企业科目代码
     * @param accountSetId - 企业账套ID
     * @param industryID - 行业ID
     * @param orgID - 机构ID
     * @return
     */
    private List<EnterpriseAccount> getMappingResultByParentCode(String parentCode, String accountSetID, String industryID, String orgID) {
        return accountMapper.getMappingResult(accountSetID, industryID, orgID, parentCode);
    }
    
    private void addMapParentAccountResult(String key, List<String> value)
    {
        if (!mapParentAccountResult.containsKey(key))
        {
            this.mapParentAccountResult.put(key, value);
        }
    }
    
    private String getConvertedIndustryID(String industryID)
    {
        if (industryID != null && industryID.equals(IndustryConstant.RealEstateID))
        {
            return industryID;
        }
        return IndustryConstant.GeneralIndustryID;
    }
    
    private String getEnterpriseAccountInfo(EnterpriseAccount enterpriseAccount)
    {
        String str = "";
        if (enterpriseAccount.getAcctProp()!=null && enterpriseAccount.getAcctProp() < EnterpriseAccountConstant.acctPropLogList.size()) {
            if (enterpriseAccount.getAcctProp() <= -1) {
                str = EnterpriseAccountConstant.acctPropLogList.get(0);
            }
            else {
                str = EnterpriseAccountConstant.acctPropLogList.get(enterpriseAccount.getAcctProp());
            }
        }
        return enterpriseAccount.getCode() + " " 
            + enterpriseAccount.getName() + " " + str + " " 
            + EnterpriseAccountConstant.directionLogMap.get(enterpriseAccount.getDirection());
    }
    
    private void enableEnterpriseAccountSet(String enterpriseAccountSetID) {
        
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey(enterpriseAccountSetID);
        if(enterpriseAccountSet.getIsActive()) {
            return;
        }
        
        EnterpriseAccountExample example = new EnterpriseAccountExample();
        example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
        List<EnterpriseAccount> enterpriseAccountList = enterpriseAccountMapper.selectByExample(example);
        
        if (Collections.isEmpty(enterpriseAccountList)) {
            return;
        }
        
        List<EnterpriseAccountDto> enterpriseAccountDtoList = new ArrayList<>();
        enterpriseAccountList.stream().forEach(sa -> {
            EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
            CommonUtils.copyProperties(sa, enterpriseAccountDto);
            enterpriseAccountDtoList.add(enterpriseAccountDto);
        });
        List<ValidateInfoDto> validateInfoDtoList = validateEnterpriseAccountList(enterpriseAccountDtoList);
        boolean anyValidationErrorExist = validateInfoDtoList.stream().filter(sa -> !Collections.isEmpty(sa.getInValidateCodeList())).findAny().isPresent();
        if(!anyValidationErrorExist) {
            if(!enterpriseAccountSet.getIsActive()) {
                enterpriseAccountSet.setIsActive(CommonConstants.ACTIVE_STATUS);
                enterpriseAccountSetMapper.updateByPrimaryKey(enterpriseAccountSet);
            }
        }
            
    }
    
    
    /* (non-Javadoc)
     * @see pwc.taxtech.atms.service.EnterpriseAccountService#validateImportEnterpriseAccount(java.io.InputStream, java.lang.String)
     */
    @Override
    public OperationResultDto<List<EnterpriseAccountDto>> validateImportEnterpriseAccount(String filePath) {
        
        //validate header
        InputStream savedInputStream = Files.findFileAsStream(filePath);
        Map<String, Integer> headerMap = fileService.readExcelHeaderAndClose(savedInputStream);
        List<String> standardHeader = Arrays.asList(
                EnterpriseAccountConstant.Code, 
                EnterpriseAccountConstant.Name, 
                EnterpriseAccountConstant.ParentCode, 
                EnterpriseAccountConstant.Direction, 
                EnterpriseAccountConstant.AcctProp);
        boolean hasParentCode = true;
        for(String header: standardHeader) {
            Integer index = headerMap.get(header);
            if(index == null) {
                if(header.equals(EnterpriseAccountConstant.ParentCode)) {
                    hasParentCode = false;
                    continue;
                }
                return new OperationResultDto<>(false, EnterpriseAccountMessage.EnterpriceAccountImportDataFormatError);
            }
        }
        
        //read excel data
        savedInputStream = Files.findFileAsStream(filePath);
        Collection<Map> mapCollection = fileService.readExcelAndClose(savedInputStream);
        
        //nothing to import
        if(mapCollection==null || Collections.isEmpty(mapCollection)) {
            return new OperationResultDto<>(true);
        }
        
        //validate data
        List<EnterpriseAccountDto> importData = new ArrayList<>();
        for (Map map: mapCollection) {
            EnterpriseAccountDto enterpriseAccountDto = new EnterpriseAccountDto();
            
            //direction
            String direction = map.get(EnterpriseAccountConstant.Direction)==null ? null: map.get(EnterpriseAccountConstant.Direction).toString();
            if (direction == null || direction.trim().isEmpty()) {
                direction = CommonConstants.DebitDirection;
            }
            try {
                enterpriseAccountDto.setDirection(Integer.valueOf(direction));
            }
            catch (NumberFormatException e) {
                return new OperationResultDto<>(false, EnterpriseAccountMessage.DirectionFormatError);
            }
            
            //accprop 
            String acctProp = map.get(EnterpriseAccountConstant.AcctProp)==null ? null:map.get(EnterpriseAccountConstant.AcctProp).toString();
            if(acctProp == null || acctProp.trim().isEmpty()) {
                acctProp = "-1";
            }
            try {
                enterpriseAccountDto.setAcctProp(Integer.valueOf(acctProp));
            }
            catch (NumberFormatException e) {
                return new OperationResultDto<>(false, EnterpriseAccountMessage.AcctPropFormatError);
            }
            
            //code
            String code = map.get(EnterpriseAccountConstant.Code)==null ? null:map.get(EnterpriseAccountConstant.Code).toString();
            if(code == null || code.trim().isEmpty()) {
                return new OperationResultDto<>(false, EnterpriseAccountMessage.CodeEmpty);
            }
            if(code.length()>EnterpriseAccountConstant.CodeMaxLength) {
                return new OperationResultDto<>(false, EnterpriseAccountMessage.CodeMaxLength);
            }
            enterpriseAccountDto.setCode(code);
            
            //name
            String name = map.get(EnterpriseAccountConstant.Name)==null ? null:map.get(EnterpriseAccountConstant.Name).toString();
            if(name==null || name.trim().isEmpty()) {
                name = "";
            }
            if(name.length()>EnterpriseAccountConstant.NameMaxLength) {
                return new OperationResultDto<>(false, EnterpriseAccountMessage.NameMaxLength);
            }
            enterpriseAccountDto.setName(name);
            
            //parentCode
            String parentCode = "";
            if(hasParentCode) {
                parentCode = map.get(EnterpriseAccountConstant.ParentCode)==null ? null:map.get(EnterpriseAccountConstant.ParentCode).toString();
            }
            if(enterpriseAccountDto.getCode().equals(parentCode)) {
                parentCode = "";
            }
            enterpriseAccountDto.setParentCode(parentCode);
        
            importData.add(enterpriseAccountDto);
        }
        OperationResultDto<List<EnterpriseAccountDto>> result = new OperationResultDto<>(true); 
        result.setData(importData);
        return result;
    }
    
    private OperationResultDto<?> importDataProcess(List<EnterpriseAccountDto> importData, String enterpriseAccountSetID, boolean isImportAppend) {
        
        //nothing to import
        if(Collections.isEmpty(importData) && isImportAppend) {
            return OperationResultDto.success();
        }
        // transfrom data
        List<EnterpriseAccount> importEntityList = transform(importData, enterpriseAccountSetID);
        
        //disable EnterpriseAccountSet
        EnterpriseAccountSet enterpriseAccountSet = enterpriseAccountSetMapper.selectByPrimaryKey(enterpriseAccountSetID);
        if(enterpriseAccountSet.getIsActive()) {
            enterpriseAccountSet.setIsActive(CommonConstants.DEACTIVE_STATUS);
            enterpriseAccountSetMapper.updateByPrimaryKey(enterpriseAccountSet);
        }

        //import
        //如覆盖导入先批量删除
        if(!isImportAppend) {
            EnterpriseAccountExample example = new EnterpriseAccountExample();
            example.createCriteria().andEnterpriseAccountSetIDEqualTo(enterpriseAccountSetID);
            enterpriseAccountMapper.deleteByExample(example);
        }
        //导入
        for(EnterpriseAccount importEntity: importEntityList) {
            enterpriseAccountMapper.insertSelective(importEntity);
        }
        
        //enable EnterpriseAccountSet
        enableEnterpriseAccountSet(enterpriseAccountSetID);
        
        return OperationResultDto.success();
        
    }
    
    /**
     * @param importData
     * @param enterpriseAccountSetID
     * @return
     */
    private List<EnterpriseAccount> transform(List<EnterpriseAccountDto> importData, String enterpriseAccountSetID) {
        
        if(Collections.isEmpty(importData)) {
            return new ArrayList<>();
        }
        importData.stream().forEach(account -> {
            account.setID(CommonUtils.getUUID());
            account.setEnterpriseAccountSetID(enterpriseAccountSetID);
            account.setIsActive(true);
            account.setCreateTime(new Date());
            account.setUpdateTime(new Date());
            String userId = authUserHelper.getCurrentUserID();
            account.setCreatorID(userId);
            account.setUpdatorID(userId);
        });
        
        //set parent code, 
        List<String> validateCodeList = importData.stream().map(sa -> sa.getCode()).collect(Collectors.toList());
        for (EnterpriseAccountDto enterpriseAccountDto: importData) {
            if(enterpriseAccountDto.getParentCode()==null || enterpriseAccountDto.getParentCode().isEmpty()) {
                enterpriseAccountDto.setParentCode(getMatchParentCode(enterpriseAccountDto.getCode(), validateCodeList));
            }
        }
        
        //set acct level,is leaf
        List<String> parentCodeList = importData.stream().filter(sa -> sa.getParentCode()!=null && !sa.getParentCode().isEmpty())
                .map(sa -> sa.getParentCode()).collect(Collectors.toList());
        for (EnterpriseAccountDto enterpriseAccountDto: importData) {
            setAcctLevel(enterpriseAccountDto, importData);
            if(Collections.isEmpty(parentCodeList)) {
                enterpriseAccountDto.setIsLeaf(true);
            }
            else {
                if(parentCodeList.contains(enterpriseAccountDto.getCode())) {
                    enterpriseAccountDto.setIsLeaf(false);
                }
                else {
                    enterpriseAccountDto.setIsLeaf(true);
                }
            }
        }
        
        //populate entity list
        List<EnterpriseAccount> enterpriseAccountList = new ArrayList<>();
        importData.stream().forEach(enterpriseAccountDto -> {
            EnterpriseAccount enterpriseAccount = new EnterpriseAccount();
            CommonUtils.copyProperties(enterpriseAccountDto, enterpriseAccount);
            enterpriseAccount.setRuleType(2);
            enterpriseAccountList.add(enterpriseAccount);
        });
        
        return enterpriseAccountList;
    }
    
    
    /**
     * Get parent Code from existing validated code list
     * @param code
     * @param validateCodeList
     * @return parent code
     */
    private String getMatchParentCode(String code, List<String> validateCodeList) {
        
        for(int i = code.length()-1; i>=0; i--) {
            String temp = code.substring(0, i);
            List<String> matchList = validateCodeList.stream().filter(vCode -> vCode.equals(temp)).collect(Collectors.toList());
            if(matchList!=null && matchList.size()==1) {
                return temp;
            }
        }
        return null;
    }
    
    /**
     * Sets the acct level.
     * @param current - The current
     * @param validateList - The validate list
     */
    private void setAcctLevel(EnterpriseAccountDto currentDto, List<EnterpriseAccountDto> validateDtoList) {
        
        if(currentDto.getParentCode()==null || currentDto.getParentCode().isEmpty()) {
            currentDto.setAcctLevel(1);
            currentDto.setFullName(currentDto.getName());
            return;
        }
        EnterpriseAccountDto parentDto = null;
        Optional<EnterpriseAccountDto> ifparentDtoExist = validateDtoList.stream()
                .filter(sa -> sa.getCode().equals(currentDto.getParentCode())).findAny();
        if(ifparentDtoExist.isPresent()) {
            parentDto = ifparentDtoExist.get();
        }
        if(parentDto!=null) {
            if(parentDto.getAcctLevel()==null) {
                setAcctLevel(parentDto, validateDtoList);
            }
            currentDto.setAcctLevel(parentDto.getAcctLevel()+1);
            
            if(currentDto.getName()!=null && !currentDto.getName().isEmpty() 
                    && !currentDto.getName().contains(parentDto.getFullName() + EnterpriseAccountConstant.FullNameSeparator)) {
                currentDto.setFullName(parentDto.getFullName() + EnterpriseAccountConstant.FullNameSeparator + currentDto.getName());
            }
            else {
                currentDto.setFullName(currentDto.getName());
            }
        }
        else {
            currentDto.setFullName(currentDto.getName());
        }
    }
    
}
